from .authenticator import Authenticator
import requests
import json


class StackService(Authenticator):
    def __init__(self, service_url, client_key_id, shared_secret, **kwargs):
        self.service_url = service_url
        self.service_token = 'StackService'
        super().__init__(client_key_id, shared_secret, **kwargs)

    def installations_list(self):
        path = '/installation-management/installations'

        bearer_token = self.get_service_bearer_token(self.service_token);
        r = requests.get(self.service_url + path, headers={'Content-Type': 'application/json', 'Authorization': bearer_token})
        return(json.loads(r.text))

    def backup_installation_to_bucket(self, installation, signed_metadata_url, signed_filedata_url, signed_dbdata_url, callback_url):
        path = '/installation-management/installation/%s/backup' % installation
        payload = {'filedata_s3_url': signed_filedata_url, 'manifest_s3_url': signed_metadata_url, 'database_s3_url': signed_dbdata_url, 'callback_url': callback_url}

        bearer_token = self.get_service_bearer_token(self.service_token);
        r = requests.post(self.service_url + path, data=json.dumps(payload), headers={'Content-Type': 'application/json', 'Authorization': bearer_token})
        return(json.loads(r.text))

    def web_server_status(self):
        path = '/status'
        bearer_token = self.get_service_bearer_token(self.service_token);
        r = requests.get(self.service_url + path, headers={'Content-Type': 'application/json', 'Authorization': bearer_token})
        return(json.loads(r.text))

    def web_server_reload(self):
        path = '/web/reload'
        bearer_token = self.get_service_bearer_token(self.service_token);
        r = requests.post(self.service_url + path, headers={'Content-Type': 'application/json', 'Authorization': bearer_token})
        return(json.loads(r.text))

    def maintenance_script_list(self):
        path = '/maintenance/list_scripts'
        bearer_token = self.get_service_bearer_token(self.service_token);
        r = requests.get(self.service_url + path, headers={'Content-Type': 'application/json', 'Authorization': bearer_token})
        return(json.loads(r.text))

    def maintenance_script_execute(self, installation_name, context, script):
        path = '/installation-management/installation/%s/execute_script' % (installation_name)

        if not isinstance(script, str):
            raise ValueError('Script argument must be a string')

        if not isinstance(context, dict):
            raise ValueError('Context argument must be a dict')

        payload = {'script': script, 'context': context}
        bearer_token = self.get_service_bearer_token(self.service_token);
        r = requests.post(self.service_url + path, data=json.dumps(payload), headers={'Content-Type': 'application/json', 'Authorization': bearer_token})
        return(json.loads(r.text))
